@@ -19,6 +19,8 @@ static int fio_mmapio_queue(struct thread_data *td, struct io_u *io_u)
 	struct fio_file *f = io_u->file;
 	unsigned long long real_off = io_u->offset - f->file_offset;
 
+	fio_ro_check(td, io_u);
+
 	if (io_u->ddir == DDIR_READ)
 		memcpy(io_u->xfer_buf, f->mmap + real_off, io_u->xfer_buflen);
 	else if (io_u->ddir == DDIR_WRITE)
@@ -60,6 +62,12 @@ static int fio_mmapio_open(struct thread_data *td, struct fio_file *f)
 	if (ret)
 		return ret;
 
+	/*
+	 * for size checkup, don't mmap anything.
+	 */
+	if (!f->io_size)
+		return 0;
+
 	if (td_rw(td))
 		flags = PROT_READ | PROT_WRITE;
 	else if (td_write(td)) {
