@@ -17,7 +17,7 @@
 #include "fio.h"
 #include "parse.h"
 
-static char fio_version_string[] = "fio 1.17.2";
+static char fio_version_string[] = "fio 1.18.1";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -43,6 +43,8 @@ static int write_lat_log = 0;
 
 static int prev_group_jobs;
 
+unsigned long fio_debug = 0;
+
 /*
  * Command line options. These will contain the above, plus a few
  * extra that only pertain to fio itself and not jobs.
@@ -103,6 +105,11 @@ static struct option long_options[FIO_NR_OPTIONS] = {
 		.has_arg	= required_argument,
 		.val		= 'e',
 	},
+	{
+		.name		= "debug",
+		.has_arg	= required_argument,
+		.val		= 'd',
+	},
 	{
 		.name		= NULL,
 	},
@@ -206,13 +213,6 @@ static int fixup_options(struct thread_data *td)
 		o->write_iolog_file = NULL;
 	}
 
-	if (td->io_ops->flags & FIO_SYNCIO)
-		o->iodepth = 1;
-	else {
-		if (!o->iodepth)
-			o->iodepth = o->open_files;
-	}
-
 	/*
 	 * only really works for sequential io for now, and with 1 file
 	 */
@@ -293,6 +293,13 @@ static int fixup_options(struct thread_data *td)
 		o->time_based = 0;
 	}
 
+	if (o->fill_device && !o->size)
+		o->size = ULONG_LONG_MAX;
+	
+	if (td_rw(td) && td->o.verify != VERIFY_NONE)
+		log_info("fio: mixed read/write workload with verify. May not "
+		 "work as expected, unless you pre-populated the file\n");
+
 	return 0;
 }
 
@@ -751,9 +758,11 @@ static int setup_thread_area(void)
 	return 0;
 }
 
-static void usage(void)
+static void usage(const char *name)
 {
 	printf("%s\n", fio_version_string);
+	printf("%s [options] [job options] <job file(s)>\n", name);
+	printf("\t--debug=options\tEnable debug logging\n");
 	printf("\t--output\tWrite output to file\n");
 	printf("\t--timeout\tRuntime in seconds\n");
 	printf("\t--latency-log\tGenerate per-job latency logs\n");
@@ -767,6 +776,63 @@ static void usage(void)
 	printf("\t          \tMay be \"always\", \"never\" or \"auto\"\n");
 }
 
+#ifdef FIO_INC_DEBUG
+struct debug_level debug_levels[] = {
+	{ .name = "process",	.shift = FD_PROCESS, },
+	{ .name = "file",	.shift = FD_FILE, },
+	{ .name = "io",		.shift = FD_IO, },
+	{ .name = "mem",	.shift = FD_MEM, },
+	{ .name = "blktrace",	.shift = FD_BLKTRACE },
+	{ .name = "verify",	.shift = FD_VERIFY },
+	{ },
+};
+
+static void set_debug(const char *string)
+{
+	struct debug_level *dl;
+	char *p = (char *) string;
+	char *opt;
+	int i;
+
+	if (!strcmp(string, "?") || !strcmp(string, "help")) {
+		int i;
+
+		log_info("fio: dumping debug options:");
+		for (i = 0; debug_levels[i].name; i++) {
+			dl = &debug_levels[i];
+			log_info("%s,", dl->name);
+		}
+		log_info("all\n");
+		return;
+	} else if (!strcmp(string, "all")) {
+		fio_debug = ~0UL;
+		return;
+	}
+
+	while ((opt = strsep(&p, ",")) != NULL) {
+		int found = 0;
+
+		for (i = 0; debug_levels[i].name; i++) {
+			dl = &debug_levels[i];
+			if (!strncmp(opt, dl->name, strlen(opt))) {
+				log_info("fio: set debug option %s\n", opt);
+				found = 1;
+				fio_debug |= (1UL << dl->shift);
+				break;
+			}
+		}
+
+		if (!found)
+			log_err("fio: debug mask %s not found\n", opt);
+	}
+}
+#else
+static void set_debug(const char *string)
+{
+	log_err("fio: debug tracing not included in build\n");
+}
+#endif
+
 static int parse_cmd_line(int argc, char *argv[])
 {
 	struct thread_data *td = NULL;
@@ -795,7 +861,7 @@ static int parse_cmd_line(int argc, char *argv[])
 			terse_output = 1;
 			break;
 		case 'h':
-			usage();
+			usage(argv[0]);
 			exit(0);
 		case 'c':
 			exit(fio_show_option_help(optarg));
@@ -814,6 +880,9 @@ static int parse_cmd_line(int argc, char *argv[])
 			else if (!strcmp("never", optarg))
 				eta_print = FIO_ETA_NEVER;
 			break;
+		case 'd':
+			set_debug(optarg);
+			break;
 		case FIO_GETOPT_JOB: {
 			const char *opt = long_options[lidx].name;
 			char *val = optarg;
@@ -868,7 +937,6 @@ static int parse_cmd_line(int argc, char *argv[])
 	return ini_idx;
 }
 
-
 int parse_options(int argc, char *argv[])
 {
 	int job_files, i;
@@ -901,6 +969,7 @@ int parse_options(int argc, char *argv[])
 			return 0;
 
 		log_err("No jobs defined(s)\n");
+		usage(argv[0]);
 		return 1;
 	}
 
