@@ -44,7 +44,7 @@ static void mark_random_map(struct thread_data *td, struct io_u *io_u)
 	unsigned int blocks;
 	unsigned int nr_blocks;
 
-	block = io_u->offset / (unsigned long long) min_bs;
+	block = (io_u->offset - f->file_offset) / (unsigned long long) min_bs;
 	blocks = 0;
 	nr_blocks = (io_u->buflen + min_bs - 1) / min_bs;
 
@@ -55,7 +55,7 @@ static void mark_random_map(struct thread_data *td, struct io_u *io_u)
 		 * If we have a mixed random workload, we may
 		 * encounter blocks we already did IO to.
 		 */
-		if (!td->o.ddir_nr == 1 && !random_map_free(td, f, block))
+		if ((td->o.ddir_nr == 1) && !random_map_free(td, f, block))
 			break;
 
 		idx = RAND_MAP_IDX(td, f, block);
@@ -72,11 +72,24 @@ static void mark_random_map(struct thread_data *td, struct io_u *io_u)
 		io_u->buflen = blocks * min_bs;
 }
 
+static inline unsigned long long last_block(struct thread_data *td,
+					    struct fio_file *f,
+					    enum fio_ddir ddir)
+{
+	unsigned long long max_blocks;
+
+	max_blocks = f->io_size / td->o.min_bs[ddir];
+	if (!max_blocks)
+		return 0;
+
+	return max_blocks - 1;
+}
+
 /*
  * Return the next free block in the map.
  */
 static int get_next_free_block(struct thread_data *td, struct fio_file *f,
-			       unsigned long long *b)
+			       enum fio_ddir ddir, unsigned long long *b)
 {
 	int i;
 
@@ -85,6 +98,8 @@ static int get_next_free_block(struct thread_data *td, struct fio_file *f,
 	while ((*b) * td->o.rw_min_bs < f->real_file_size) {
 		if (f->file_map[i] != -1UL) {
 			*b += fio_ffz(f->file_map[i]);
+			if (*b > last_block(td, f, ddir))
+				break;
 			f->last_free_lookup = i;
 			return 0;
 		}
@@ -93,22 +108,20 @@ static int get_next_free_block(struct thread_data *td, struct fio_file *f,
 		i++;
 	}
 
+	dprint(FD_IO, "failed finding a free block\n");
 	return 1;
 }
 
 static int get_next_rand_offset(struct thread_data *td, struct fio_file *f,
-				int ddir, unsigned long long *b)
+				enum fio_ddir ddir, unsigned long long *b)
 {
-	unsigned long long max_blocks = f->io_size / td->o.min_bs[ddir];
 	unsigned long long r, rb;
 	int loops = 5;
 
 	do {
 		r = os_random_long(&td->random_state);
-		if (!max_blocks)
-			*b = 0;
-		else
-			*b = ((max_blocks - 1) * r / (unsigned long long) (RAND_MAX+1.0));
+		*b = last_block(td, f, ddir);
+
 		/*
 		 * if we are not maintaining a random map, we are done.
 		 */
@@ -118,7 +131,7 @@ static int get_next_rand_offset(struct thread_data *td, struct fio_file *f,
 		/*
 		 * calculate map offset and chec if it's free
 		 */
-		rb = *b + (f->file_offset / td->o.min_bs[ddir]);
+		rb = *b;
 		if (random_map_free(td, f, rb))
 			return 0;
 
@@ -132,7 +145,7 @@ static int get_next_rand_offset(struct thread_data *td, struct fio_file *f,
 	loops = 10;
 	do {
 		f->last_free_lookup = (f->num_maps - 1) * (r / (RAND_MAX+1.0));
-		if (!get_next_free_block(td, f, b))
+		if (!get_next_free_block(td, f, ddir, b))
 			return 0;
 
 		r = os_random_long(&td->random_state);
@@ -142,7 +155,7 @@ static int get_next_rand_offset(struct thread_data *td, struct fio_file *f,
 	 * that didn't work either, try exhaustive search from the start
 	 */
 	f->last_free_lookup = 0;
-	return get_next_free_block(td, f, b);
+	return get_next_free_block(td, f, ddir, b);
 }
 
 /*
@@ -153,8 +166,8 @@ static int get_next_rand_offset(struct thread_data *td, struct fio_file *f,
 static int get_next_offset(struct thread_data *td, struct io_u *io_u)
 {
 	struct fio_file *f = io_u->file;
-	const int ddir = io_u->ddir;
 	unsigned long long b;
+	enum fio_ddir ddir = io_u->ddir;
 
 	if (td_random(td) && (td->o.ddir_nr && !--td->ddir_nr)) {
 		td->ddir_nr = td->o.ddir_nr;
@@ -163,15 +176,19 @@ static int get_next_offset(struct thread_data *td, struct io_u *io_u)
 			return 1;
 	} else {
 		if (f->last_pos >= f->real_file_size) {
-			if (!td_random(td) || get_next_rand_offset(td, f, ddir, &b))
+			if (!td_random(td) ||
+			     get_next_rand_offset(td, f, ddir, &b))
 				return 1;
 		} else
 			b = (f->last_pos - f->file_offset) / td->o.min_bs[ddir];
 	}
 
 	io_u->offset = (b * td->o.min_bs[ddir]) + f->file_offset;
-	if (io_u->offset >= f->real_file_size)
+	if (io_u->offset >= f->real_file_size) {
+		dprint(FD_IO, "get_next_offset: offset %llu >= size %llu\n",
+					io_u->offset, f->real_file_size);
 		return 1;
+	}
 
 	return 0;
 }
@@ -186,13 +203,30 @@ static unsigned int get_next_buflen(struct thread_data *td, struct io_u *io_u)
 		buflen = td->o.min_bs[ddir];
 	else {
 		r = os_random_long(&td->bsrange_state);
-		buflen = (unsigned int) (1 + (double) (td->o.max_bs[ddir] - 1) * r / (RAND_MAX + 1.0));
+		if (!td->o.bssplit_nr)
+			buflen = (unsigned int) (1 + (double) (td->o.max_bs[ddir] - 1) * r / (RAND_MAX + 1.0));
+		else {
+			long perc = 0;
+			unsigned int i;
+
+			for (i = 0; i < td->o.bssplit_nr; i++) {
+				struct bssplit *bsp = &td->o.bssplit[i];
+
+				buflen = bsp->bs;
+				perc += bsp->perc;
+				if (r <= ((LONG_MAX / 100L) * perc))
+					break;
+			}
+		}
 		if (!td->o.bs_unaligned)
 			buflen = (buflen + td->o.min_bs[ddir] - 1) & ~(td->o.min_bs[ddir] - 1);
 	}
 
-	if (io_u->offset + buflen > io_u->file->real_file_size)
+	if (io_u->offset + buflen > io_u->file->real_file_size) {
+		dprint(FD_IO, "lower buflen %u -> %u (ddir=%d)\n", buflen,
+						td->o.min_bs[ddir], ddir);
 		buflen = td->o.min_bs[ddir];
+	}
 
 	return buflen;
 }
@@ -303,6 +337,9 @@ void requeue_io_u(struct thread_data *td, struct io_u **io_u)
 	struct io_u *__io_u = *io_u;
 
 	__io_u->flags |= IO_U_F_FREE;
+	if ((__io_u->flags & IO_U_F_FLIGHT) && (__io_u->ddir != DDIR_SYNC))
+		td->io_issues[__io_u->ddir]--;
+		
 	__io_u->flags &= ~IO_U_F_FLIGHT;
 
 	list_del(&__io_u->list);
@@ -341,15 +378,23 @@ static int fill_io_u(struct thread_data *td, struct io_u *io_u)
 	 * No log, let the seq/rand engine retrieve the next buflen and
 	 * position.
 	 */
-	if (get_next_offset(td, io_u))
+	if (get_next_offset(td, io_u)) {
+		dprint(FD_IO, "io_u %p, failed getting offset\n", io_u);
 		return 1;
+	}
 
 	io_u->buflen = get_next_buflen(td, io_u);
-	if (!io_u->buflen)
+	if (!io_u->buflen) {
+		dprint(FD_IO, "io_u %p, failed getting buflen\n", io_u);
 		return 1;
+	}
 
-	if (io_u->offset + io_u->buflen > io_u->file->real_file_size)
+	if (io_u->offset + io_u->buflen > io_u->file->real_file_size) {
+		dprint(FD_IO, "io_u %p, offset too large\n", io_u);
+		dprint(FD_IO, "  off=%llu/%lu > %llu\n", io_u->offset,
+				io_u->buflen, io_u->file->real_file_size);
 		return 1;
+	}
 
 	/*
 	 * mark entry before potentially trimming io_u
@@ -361,6 +406,8 @@ static int fill_io_u(struct thread_data *td, struct io_u *io_u)
 	 * If using a write iolog, store this entry.
 	 */
 out:
+	dprint_io_u(io_u, "fill_io_u");
+	td->zone_bytes += io_u->buflen;
 	log_io_u(td, io_u);
 	return 0;
 }
@@ -509,8 +556,10 @@ static struct fio_file *get_next_file_rand(struct thread_data *td, int goodf,
 		if (f->flags & FIO_FILE_DONE)
 			continue;
 
-		if ((!goodf || (f->flags & goodf)) && !(f->flags & badf))
+		if ((!goodf || (f->flags & goodf)) && !(f->flags & badf)) {
+			dprint(FD_FILE, "get_next_file_rand: %p\n", f);
 			return f;
+		}
 	} while (1);
 }
 
@@ -541,6 +590,7 @@ static struct fio_file *get_next_file_rr(struct thread_data *td, int goodf,
 		f = NULL;
 	} while (td->next_file != old_next_file);
 
+	dprint(FD_FILE, "get_next_file_rr: %p\n", f);
 	return f;
 }
 
@@ -550,12 +600,14 @@ static struct fio_file *get_next_file(struct thread_data *td)
 
 	assert(td->o.nr_files <= td->files_index);
 
-	if (!td->nr_open_files || td->nr_done_files >= td->o.nr_files)
+	if (!td->nr_open_files || td->nr_done_files >= td->o.nr_files) {
+		dprint(FD_FILE, "get_next_file: nr_open=%d, nr_done=%d, nr_files=%d\n", td->nr_open_files, td->nr_done_files, td->o.nr_files);
 		return NULL;
+	}
 
 	f = td->file_service_file;
 	if (f && (f->flags & FIO_FILE_OPEN) && td->file_service_left--)
-		return f;
+		goto out;
 
 	if (td->o.file_service_type == FIO_FSERVICE_RR)
 		f = get_next_file_rr(td, FIO_FILE_OPEN, FIO_FILE_CLOSING);
@@ -564,6 +616,8 @@ static struct fio_file *get_next_file(struct thread_data *td)
 
 	td->file_service_file = f;
 	td->file_service_left = td->file_service_nr - 1;
+out:
+	dprint(FD_FILE, "get_next_file: %p\n", f);
 	return f;
 }
 
@@ -664,8 +718,10 @@ struct io_u *get_io_u(struct thread_data *td)
 	struct io_u *io_u;
 
 	io_u = __get_io_u(td);
-	if (!io_u)
+	if (!io_u) {
+		dprint(FD_IO, "__get_io_u failed\n");
 		return NULL;
+	}
 
 	/*
 	 * from a requeue, io_u already setup
@@ -679,15 +735,19 @@ struct io_u *get_io_u(struct thread_data *td)
 	if (td->o.read_iolog_file) {
 		if (read_iolog_get(td, io_u))
 			goto err_put;
-	} else if (set_io_u_file(td, io_u))
+	} else if (set_io_u_file(td, io_u)) {
+		dprint(FD_IO, "io_u %p, setting file failed\n", io_u);
 		goto err_put;
+	}
 	
 	f = io_u->file;
 	assert(f->flags & FIO_FILE_OPEN);
 
 	if (io_u->ddir != DDIR_SYNC) {
-		if (!io_u->buflen)
+		if (!io_u->buflen && !(td->io_ops->flags & FIO_NOIO)) {
+			dprint(FD_IO, "get_io_u: zero buflen on %p\n", io_u);
 			goto err_put;
+		}
 
 		f->last_pos = io_u->offset + io_u->buflen;
 
@@ -708,6 +768,7 @@ out:
 		return io_u;
 	}
 err_put:
+	dprint(FD_IO, "get_io_u failed\n");
 	put_io_u(td, io_u);
 	return NULL;
 }
@@ -734,6 +795,8 @@ static void io_completed(struct thread_data *td, struct io_u *io_u,
 {
 	unsigned long usec;
 
+	dprint_io_u(io_u, "io complete");
+
 	assert(io_u->flags & IO_U_F_FLIGHT);
 	io_u->flags &= ~IO_U_F_FLIGHT;
 
@@ -751,7 +814,6 @@ static void io_completed(struct thread_data *td, struct io_u *io_u,
 
 		td->io_blocks[idx]++;
 		td->io_bytes[idx] += bytes;
-		td->zone_bytes += bytes;
 		td->this_io_bytes[idx] += bytes;
 
 		io_u->file->last_completed_pos = io_u->endpos;
@@ -763,6 +825,7 @@ static void io_completed(struct thread_data *td, struct io_u *io_u,
 		io_u_mark_latency(td, usec);
 
 		if (td_write(td) && idx == DDIR_WRITE &&
+		    td->o.do_verify &&
 		    td->o.verify != VERIFY_NONE)
 			log_io_piece(td, io_u);
 
@@ -831,6 +894,8 @@ long io_u_queued_complete(struct thread_data *td, int min_events)
 	int ret;
 	struct timespec ts = { .tv_sec = 0, .tv_nsec = 0, };
 
+	dprint(FD_IO, "io_u_queued_completed: min=%d\n", min_events);
+
 	if (!min_events)
 		tvp = &ts;
 
